<?php namespace ProcessWire;

require_once "MainController.php";

/**
 * Class FormDemoController
 * @package ProcessWire
 */
Class FormDemoController extends MainController {

	/**
	 * Set template variables
	 * @return void
	 * @throws \ProcessWire\WireException
	 */

	protected $values = [
		"company" => "",
		"city" => "",
		"email" => "",
		"prlanguages" => "",
		"language" => "",
		"date" => "",
		"message" => "",
	];

	public function render(): void {

		//we always create a form. Either it will be empty, or filled with given values.
		$form = $this->createForm();

		//we check if we have a submitted form. The submit button is a great choice to check upon,
		//but it could have been another field or a Cancel button.

		if(input()->post("pf_submit_btn")) {

			//first we must validate
			//processInput can be hooked for further validation.
			//if any error found, the $form->getErrors() will tell us so.
			$form->processInput(input()->post);

			//Do we have errors?
			if(count($form->getErrors()) > 0) {
				//we found errors, we display again the form to correct the situation
				//the form is populated with the error messages above the appropriate fields
				$this->view->set("form", $form->render());
			} else {
				//no preliminary error found. we can process.
				$this->values["company"] = sanitizer()->input()->post("company");
				$this->values["city"] = sanitizer()->input()->post("city");
				$this->values["email"] = sanitizer()->input()->post("email");
				$this->values["prlanguages"] = sanitizer()->input()->post("prlanguages");
				$this->values["language"] = sanitizer()->input()->post("language");
				$this->values["date"] = sanitizer()->input()->post("date");
				$this->values["message"] = sanitizer()->input()->post("message");

				//does the values satisfies us?
				//if not, work on it and send again the form to the user!
				//here the remaining of the processsing code.
			}

		} else {
			//we have no input, we must ask for it.
			//The form is by default empty (except for the date in this example)
			//We use here the twig template engine
			$this->view->set("form", $form->render());
		}
	}

	protected function createForm() {

		//Since the form is empty, all values are.
		//But this form could be populated with previous values if, for
		//example, the user has made mistakes.
		if(empty($this->values)) {
			$this->values = [
				"company" => "",
				"city" => "",
				"email" => "",
				"prlanguages" => "",
				"language" => "",
				"date" => "",
			];
		}
		// Create our form
		$form = modules()->get("InputfieldForm");
		$form->action = "";
		$form->method = "post";
		$form->attr("name+id", "pro_form");
		$form->attr("class", "uk-form-stacked no-form-ul");

		//CSRF
		$f = modules()->get("InputfieldHidden");
		$f->attr("id+name", session()->CSRF->getTokenName());
		$f->attr("maxlength", 30);
		$f->attr("class", "uk-input");
		$f->attr("value", session()->CSRF->getTokenValue());

		//Markup All fields are required
		$markup = modules()->get('InputfieldMarkup');
		$markup->value = "<div class='warning'>" . __("All fields are required") . "</div>";
		$form->add($markup);


		//Our form will be split into two columns,
		//We create fieldset to do so.
		$firstCol = modules()->get("InputfieldFieldset");
		$firstCol->attr("class", "firstCol");
		$form->add($firstCol);

		$secondCol = modules()->get("InputfieldFieldset");
		$secondCol->attr("class", "secondCol");
		$form->add($secondCol);

		// Text input
		$f = modules()->get("InputfieldText");
		$f->set("label", __("Designer’s / architect’s office"));
		$f->attr("name+id", "company");
		$f->attr("maxlength", 50);
		$f->attr("class", "uk-form-width-medium uk-input");
		$f->attr("value", $this->values["company"]);
		$f->required(true);
		$f->attr("placeholder", __('Name of the company'));
		$firstCol->add($f);  // could also have been => $form->add($f)

		//Select input
		//Options can be given by the PW database.
		$options = [
			'montreal' => __("Montreal"),
			'quebec' => __("Quebec City")
		];

		$f = modules()->get("InputfieldSelect");
		$f->set("label", __("City"));
		$f->attr("name+id", "city");
		$f->attr("maxlength", 50);
		$f->attr("class", "uk-form-width-medium uk-select");
		$f->attr("value", $this->values["city"]);
		$f->addOptions($options);
		$f->required(true);
		$firstCol->add($f);

		//Email input
		$f = wire("modules")->get("InputfieldEmail");
		$f->set("label", __("Email"));
		$f->set('confirm', 1);
		$f->attr("name+id", "email");
		$f->attr("maxlength", 60);
		$f->attr("class", "uk-form-width-medium");
		$f->attr("value", $this->values["email"]);
		$f->required = true;
		$firstCol->add($f);

		//Select input with autocomplete
		//the values may come from PW database.
		//these options are not passed into the field but to the template engine
		$optionsLanguages = '
				"ActionScript",
				"AppleScript",
				"Asp",
				"BASIC",
				"C",
				"C++",
				"Clojure",
				"COBOL",
				"ColdFusion",
				"Erlang",
				"Fortran",
				"Groovy",
				"Haskell",
				"Java",
				"JavaScript",
				"Lisp",
				"Perl",
				"PHP",
				"Python",
				"Ruby",
				"Scala",
				"Scheme"';
		$f = modules()->get("InputfieldText");
		$f->set("label", __("Programming languages"));
		$f->attr("name+id", "prlanguages");
		$f->attr("maxlength", 50);
		$f->attr("class", "uk-form-width-medium");
		$f->attr("value", $this->values["prlanguages"]);
		$f->wrapClass("autocomplete");
		$f->required(true);
		$secondCol->add($f);

		//Radios input
		// Options may come from PW Database
		$options = [
			'fr' => __("French"),
			'en' => __("English")
		];
		$f = modules()->get("InputfieldRadios");
		$f->set("label", __("Language"));
		$f->attr("name+id", "language");
		$f->attr("maxlength", 50);
		$f->attr("class", "uk-form-width-medium uk-radio");
		$f->attr("value", $this->values["language"]);
		$f->set('optionColumns', 1); // if 0, ul has class = InputfieldRadiosStacked. If 1, ul has class == InputfieldRadiosFloated
		$f->addOptions($options);
		$f->required(true);
		$secondCol->add($f);

		//DateTime input
		$f = modules()->get("InputfieldDatetime");
		$f->set("label", __("Date"));
		$f->attr("name+id", "date");
		$f->attr("maxlength", 50);
		$f->attr("class", "uk-form-width-medium ");
		$f->attr("value", $this->values["date"]);
		$f->datepicker = 3;  //this is a popup calendar
		$f->dateInputFormat = "Y-m-d";
		$f->attr("value", time());
		$f->required(true);
		$f->prependMarkup("<div id='datepicker'>"); //we wrap the field with a class.
		$f->appendMarkup("</div>");
		$secondCol->add($f);

		// Text area
		$f = modules()->get("InputfieldTextarea");
		$f->set("label", __("Your message"));
		$f->attr("name+id", "message");
		$f->attr("rows", 10);
		$f->attr("class", "uk-form-width-medium uk-input");
		$f->attr("value", $this->values["message"]);
		$f->required(true);
		$f->attr("placeholder", __('Your message'));
		$form->add($f);

		// Submit
		$f = modules()->get("InputfieldSubmit");
		$f->attr("id", "pf-submit-btn");
		$f->attr("name", "pf_submit_btn");
		$f->attr("value", __("Continue"));
		$f->attr("class", "blue-btn large");
		$form->add($f);

		//Options sent to the twig template for javascript autocomplete
		$this->view->set("optionsLanguages", $optionsLanguages);

		return $form;
	}

}

(new FormDemoController())->render();